/* ------------------------------------------------------------------------- */
/*
 *  args.h
 *
 *  Copyright (c) 2004 - 2005 "clown". All rights reserved.
 *
 *  Permission to copy, use, modify and distribute this software is
 *  granted provided this copyright notice appears in all copies.
 *  This software is provided "as is" without express or implied
 *  warranty, and with no claim as to its suitability for any purpose.
 *
 *  Last-modified: Mon 27 Feb 2006 20:13:00 JST
 */
/* ------------------------------------------------------------------------- */
#ifndef CLX_ARGS_H
#define CLX_ARGS_H

#include <string>
#include <vector>
#include <map>
#include "clx/lexical_cast.h"

namespace clx {
	/* --------------------------------------------------------------------- */
	//  basic_args
	/* --------------------------------------------------------------------- */
	template <
		class CharT,
		class Traits = std::char_traits<CharT>
	>
	class basic_args : public std::map<std::basic_string<CharT, Traits>, 
		std::vector<std::basic_string<CharT, Traits> > >
	{
	public:
		typedef CharT char_type;
		typedef int int_type;
		typedef unsigned int size_type;
		
		typedef std::map<std::basic_string<CharT, Traits>, 
			std::vector<std::basic_string<CharT, Traits> > > args_map;
		typedef std::basic_string<CharT, Traits> string_type;
		typedef std::vector<string_type> container;
		typedef std::pair<string_type, container> element_type;
		typedef typename args_map::iterator iterator;
		
		// constructor and destructor
		explicit basic_args(char_type c = '-') :
			args_map(), identifier_(c), program_(), head_() {}
		
		explicit basic_args(int_type argc, char_type* argv[], char_type c = '-') :
			args_map(), identifier_(c), program_(), head_() { this->assign(argc, argv); }
		
		virtual ~basic_args() {}
		
		// accessor
		container head() { return head_; }
		string_type program() { return program_; }
		
		// operator
		basic_args& assign(int_type argc, char_type* argv[]) {
			program_ = argv[0];
			for (int_type i = 1; i < argc; i++) {
				string_type tmp(argv[i]);
				
				if (tmp.at(0) != identifier_) {
					head_.push_back(tmp);
					continue;
				}
				
				string_type key = tmp.substr(tmp.find_first_not_of(identifier_));
				container value;
				while (i + 1 < argc && argv[i+1][0] != identifier_) {
					tmp = argv[++i];
					value.push_back(tmp);
				}
				this->insert(element_type(key, value));
			}
			return *this;
		}
		
		bool is_exist(const string_type& s) const {
			if (this->find(s) != this->end()) return true;
			else return false;
		}
		
		template <class ValueT>
		basic_args& operator()(const string_type& key, ValueT& dest) {
			iterator pos = this->find(key);
			if (pos != this->end() && !pos->second.empty()) {
				dest = lexical_cast<ValueT>(pos->second.at(0));
			}
			return *this;
		}
		
		template <class ValueT, class Assign>
		basic_args& operator()(const string_type& key, ValueT& dest, Assign func) {
			iterator pos = this->find(key);
			if (pos != this->end() && !pos->second.empty()) func(pos->second, dest);
			return *this;
		}
	
	private:
		char_type identifier_;
		string_type program_;
		container head_;
	};
	
	typedef basic_args<char> args;
}

#endif // CLX_ARGS_H
