/* ------------------------------------------------------------------------- */
/*
 *  sockudp.h
 *
 *  Copyright (c) 2004 - 2006 "clown". All rights reserved.
 *
 *  Permission to copy, use, modify and distribute this software is
 *  granted provided this copyright notice appears in all copies.
 *  This software is provided "as is" without express or implied
 *  warranty, and with no claim as to its suitability for any purpose.
 *
 *  Last-modified: Sun 19 Feb 2006 13:51:00 JST
 */
/* ------------------------------------------------------------------------- */
#ifndef CLX_SOCKUDP_H
#define CLX_SOCKUDP_H

#include <cstdlib>
#include <cstring>
#include <string>
#include "socket.h"

namespace clx {
	/* --------------------------------------------------------------------- */
	//  basic_socket_udp
	/* --------------------------------------------------------------------- */
	template <
		int Type = SOCK_DGRAM,
		int Family = AF_INET
	>
	class basic_socket_udp : basic_socket<Type, Family> {
	public:
		typedef basic_socket_address<Family> socket_address;
		typedef char char_type;
		typedef std::basic_string<char_type> string_type;
		typedef unsigned int size_type;
		
		static const size_type bufsize = 4096;
		
		// constructor and destructor
		basic_socket_udp() :
			basic_socket<Type, Family>(), is_bind_(false) {
			buf_ = new char_type[bufsize];
			this->open();
		}
		
		explicit basic_socket_udp(int port) :
			basic_socket<Type, Family>(), is_bind_(false) { 
			buf_ = new char_type[bufsize];
			this->open();
			this->bind(port);
		}
		
		virtual ~basic_socket_udp() {
			delete buf_;
		}
		
		// operator
		virtual int send(const string_type& src, const socket_address& addr) {
			return ::sendto(this->socket(), src.c_str(), src.size(), 0, addr.base_data(), addr.size());
		}
		
		virtual int send(const string_type& src, const string_type& host, int port) {
			socket_address addr(host, port);
			return this->send(src, addr);
		}
		
		virtual int recv(string_type& dest, socket_address& addr) {
			if (!this->is_bind()) return -1;
			
			int len, n;
			typename socket_address::inet_type tmp;
			
			len = sizeof(tmp);
			memset(buf_, 0, bufsize);
			n = ::recvfrom(this->socket(), buf_, bufsize, 0, (struct sockaddr*)&tmp, &len);
			
			if (n > 0) dest = buf_;
			addr = tmp;
			return n;
		}
		
		virtual int recv(string_type& dest) {
			socket_address tmp;
			return this->recv(dest, tmp);
		}
		
		virtual int bind(int port) {
			addr_.assign(port);
			if (::bind(this->socket(), addr_.base_data(), addr_.size()) == -1) return -1;
			is_bind_ = true;
			return 0;
		}
		
		bool is_bind() const { return is_bind_; }
		
	private:
		socket_address addr_;
		char_type* buf_;
		bool is_bind_;
	};
	
	typedef basic_socket_udp<> sockudp;
}

#endif // CLX_SOCKUDP_H
