/* ------------------------------------------------------------------------- */
/*
 *  table.h
 *
 *  Copyright (c) 2004 - 2006 "clown", All rights reserved.
 *
 *  Permission to copy, use, modify and distribute this software is
 *  granted provided this copyright notice appears in all copies.
 *  This software is provided "as is" without express or implied
 *  warranty, and with no claim as to its suitability for any purpose.
 *
 *  Last-modified: Fri 20 Jan 2006 21:43:00 JST
 */
/* ------------------------------------------------------------------------- */
#ifndef CLX_TABLE_H
#define CLX_TABLE_H

#include <string>
#include <vector>
#include <fstream>
#include "tokenizer.h"

namespace clx {
	/* --------------------------------------------------------------------- */
	//  basic_table
	/* --------------------------------------------------------------------- */
	template <
		class Parser,
		class Container = std::vector<std::vector<std::string> >,
		class CharT = char,
		class Traits = std::char_traits<CharT>
	>
	class basic_table : public Container {
	public:
		typedef Parser func;
		typedef Container container;
		typedef typename std::basic_string<CharT, Traits> string_type;
		typedef typename std::basic_fstream<CharT, Traits> fstream;
		typedef typename container::size_type size_type;
		
		// constructor and destructor
		explicit basic_table(const func& f = func()) :
			Container(), fp_(), f_(f) { this->reset(); }
		
		explicit basic_table(const string_type& path, const func& f = func()) :
			Container(), fp_(), f_(f) {
			this->reset();
			this->open(path);
		}
		
		virtual ~basic_table() { if (fp_.is_open()) fp_.close(); }
		
		// operator
		virtual void reset() {
			Container::clear();
			f_.reset();
		}
		
		virtual bool readline() {
			string_type tmp;
			if (!std::getline(fp_, tmp)) return false;
			Container::push_back(f_(tmp));
			return true;
		}
		
		virtual bool read(size_type line) {
			for (size_type i = line; i > 0; i--) {
				if (!this->readline()) {
					if (i == line) return false; // cannot read at all.
					else break;
				}
			}
			return true;
		}
		
		virtual void read() { while (this->readline()); }
		
		void open(const string_type& path) { fp_.open(path.c_str()); }
		void close() { fp_.close(); }
		bool is_open() { return fp_.is_open(); }
		bool eof() { return fp_.eof(); }
			
	private:
		fstream fp_;
		func f_;
	};
	
	typedef basic_table<tokenizer> table;
	typedef basic_table<csv> csvtable;
	
	typedef basic_table<
		basic_tokenizer<char_separator<char>, std::string, std::vector<double> >,
		std::vector<std::vector<double> >
	> matrix;
}

#endif // CLX_TABLE_H
